"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Editor = void 0;
const Cursor_1 = require("./Cursor");
const clock_1 = require("../../../json-crdt-patch/clock");
/**
 * @todo Rename to `PeritextApi`.
 */
class Editor {
    constructor(txt, slices) {
        this.txt = txt;
        const point = txt.pointAbsStart();
        const range = txt.range(point, point.clone());
        // TODO: Add ability to remove cursor.
        this.cursor = slices.ins(range, 4 /* SliceBehavior.Cursor */, 0 /* CursorAnchor.Start */, undefined, Cursor_1.Cursor);
    }
    /** @deprecated */
    setCursor(start, length = 0) {
        this.cursor.setAt(start, length);
    }
    /** @deprecated */
    getCursorText() {
        return this.cursor.text();
    }
    /**
     * Ensures there is no range selection. If user has selected a range,
     * the contents is removed and the cursor is set at the start of the range as cursor.
     *
     * @todo If block boundaries are withing the range, remove the blocks.
     *
     * @returns Returns the cursor position after the operation.
     */
    collapseSelection() {
        const cursor = this.cursor;
        const isCaret = cursor.isCollapsed();
        if (!isCaret) {
            const { start, end } = cursor;
            const txt = this.txt;
            const deleteStartId = start.anchor === 0 /* Anchor.Before */ ? start.id : start.nextId();
            const deleteEndId = end.anchor === 1 /* Anchor.After */ ? end.id : end.prevId();
            const str = txt.str;
            if (!deleteStartId || !deleteEndId)
                throw new Error('INVALID_RANGE');
            const range = str.findInterval2(deleteStartId, deleteEndId);
            const model = txt.model;
            const api = model.api;
            api.builder.del(str.id, range);
            api.apply();
            if (start.anchor === 1 /* Anchor.After */)
                cursor.setAfter(start.id);
            else
                cursor.setAfter(start.prevId() || str.id);
        }
        return cursor.start.id;
    }
    /**
     * Insert inline text at current cursor position. If cursor selects a range,
     * the range is removed and the text is inserted at the start of the range.
     */
    insert(text) {
        if (!text)
            return;
        const after = this.collapseSelection();
        const textId = this.txt.ins(after, text);
        const shift = text.length - 1;
        this.cursor.setAfter(shift ? (0, clock_1.tick)(textId, shift) : textId);
    }
    /**
     * Deletes the previous character at current cursor position. If cursor
     * selects a range, deletes the whole range.
     */
    delete() {
        const isCollapsed = this.cursor.isCollapsed();
        if (isCollapsed) {
            const range = this.txt.findCharBefore(this.cursor.start);
            if (!range)
                return;
            this.cursor.set(range.start, range.end);
        }
        this.collapseSelection();
    }
    start() {
        const txt = this.txt;
        const str = txt.str;
        if (!str.length())
            return;
        const firstChunk = str.first();
        if (!firstChunk)
            return;
        const firstId = firstChunk.id;
        const start = txt.point(firstId, 0 /* Anchor.Before */);
        return start;
    }
    end() {
        const txt = this.txt;
        const str = txt.str;
        if (!str.length())
            return;
        const lastChunk = str.last();
        if (!lastChunk)
            return;
        const lastId = lastChunk.span > 1 ? (0, clock_1.tick)(lastChunk.id, lastChunk.span - 1) : lastChunk.id;
        const end = txt.point(lastId, 1 /* Anchor.After */);
        return end;
    }
    all() {
        const start = this.start();
        const end = this.end();
        if (!start || !end)
            return;
        return this.txt.range(start, end);
    }
    selectAll() {
        const range = this.all();
        if (range)
            this.cursor.setRange(range);
    }
    insStackSlice(type, data) {
        const range = this.cursor.range();
        return this.txt.savedSlices.ins(range, 1 /* SliceBehavior.Stack */, type, data);
    }
    insOverwriteSlice(type, data) {
        const range = this.cursor.range();
        return this.txt.savedSlices.ins(range, 2 /* SliceBehavior.Overwrite */, type, data);
    }
    insEraseSlice(type, data) {
        const range = this.cursor.range();
        return this.txt.savedSlices.ins(range, 3 /* SliceBehavior.Erase */, type, data);
    }
    insMarker(type, data) {
        const after = this.collapseSelection();
        return this.txt.insMarker(after, type, data, "\n" /* Chars.BlockSplitSentinel */);
    }
}
exports.Editor = Editor;
